/******************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlBox3d.h
** AUTHOR(S):   Paul Baumstarck <pbaumstarck@stanford.edu>
** DESCRIPTION:
**  Bounding box class for svlPoint3d's. 
*****************************************************************************/

#if defined(_WIN32)||defined(WIN32)||defined(__WIN32__)
#define _CRT_SECURE_NO_DEPRECATE
#undef max
#undef min
#endif

#ifndef _SVLBOX3D_H
#define _SVLBOX3D_H

#include <limits>
#include <iostream>

#include "../vision/svlPoint3d.h"

using namespace std;

class svlBox3d {
public:
	svlPoint3d max_corner, min_corner;
public:
	inline svlBox3d()
	    : max_corner(svlPoint3d(-numeric_limits<double>::max())),
	    min_corner(svlPoint3d(numeric_limits<double>::max())) {}
    inline svlBox3d(const svlPoint3d& pt)
	    : max_corner(svlPoint3d(pt)),
	    min_corner(svlPoint3d(pt)) {}
	inline svlBox3d(const svlBox3d& box)
	    : max_corner(box.max_corner),
	    min_corner(box.min_corner) {}
	inline svlBox3d(const svlBox3d& box1, const svlBox3d& box2)
	    : max_corner(box1.max_corner),
	    min_corner(box1.min_corner) {
	    max_corner.max_with(box2.max_corner);
	    min_corner.min_with(box2.min_corner);
	}
	virtual ~svlBox3d() {}; // do nothing

	inline void addToBox(const svlPoint3d &pt) {
		min_corner.min_with(pt);
		max_corner.max_with(pt);
	}

	inline void unionWithBox(const svlBox3d &box) {
		min_corner.min_with(box.min_corner);
		max_corner.max_with(box.max_corner);
	}

	inline double minDistFromPt(const svlPoint3d &pt) {
		return std::min( (max_corner-pt).pt_min(), (pt-min_corner).pt_min() );
	}

	inline double mDFPWOB(const svlPoint3d &pt, const svlBox3d &box) {
		return minDistFromPtWithOuterBox(pt,box);
	}

	inline double minDistFromPtWithOuterBox(const svlPoint3d &pt, const svlBox3d &box) {
		return std::min( svlPoint3d( max_corner.x == box.max_corner.x ? numeric_limits<double>::max() : max_corner.x - pt.x,
			max_corner.y == box.max_corner.y ? numeric_limits<double>::max() : max_corner.y - pt.y,
			max_corner.z == box.max_corner.z ? numeric_limits<double>::max() : max_corner.z - pt.z ).pt_min(),
		    svlPoint3d( min_corner.x == box.min_corner.x ? numeric_limits<double>::max() : pt.x - min_corner.x,
			min_corner.y == box.min_corner.y ? numeric_limits<double>::max() : pt.y - min_corner.y,
			min_corner.z == box.min_corner.z ? numeric_limits<double>::max() : pt.z - min_corner.z ).pt_min() );
	}
	
	friend std::ostream& operator<<(std::ostream& os, const svlBox3d& p) {
	    os << "(" << p.min_corner << ") -> (" << p.max_corner << ")";
	    return os;
	}
};

#endif

