/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlPlaneClustering.h
** AUTHOR(S):   Paul Baumstarck <pbaumstarck@stanford.edu>
**
** DESCRIPTION: Switchyard class for performing plane clustering.
**
*****************************************************************************/

#pragma once

#include <cstdlib>
#include <vector>
#include <map>

#include "cv.h"
#include "cxcore.h"

//#include "svlBase.h"
//#include "svlDevel.h"
#include "svlAlgorithms.h"
#include "svlSortVector.h"
#include "svlPlane.h"
#include "svlKdTreeT.h"
#include "../base/svlOptions.h"
#include "svlMapping.h"
#include "svlRabbaniClustering.h"
#include "svlVision.h"
#include "svlKMeansClustering.h"

using namespace std;

class svlPlaneClustering : public svlOptions {
public:
	// input data structures
	int w, h;
	svlAlgorithms _sva;
	svlCameraIntrinsics *_intrinsics;
	svlCameraExtrinsics *_extrinsics;
	CvMat *_X; // reconstructed X, Y, Z
	CvMat *_Y;
	CvMat *_Z;
	CvMat *_nX; // reconstructed normal X, Y, Z
	CvMat *_nY;
	CvMat *_nZ;
	CvMat *_Ix;
	
	// internal processing 
	vector<svlPoint3d> *_reconstP, *_reconstN; // reconstructed points and normals
	vector<int> _ix;
	vector<vector<int> > _regionIx;

	// output data structures
	vector<svlPlane> _planes, _camPlanes;

protected:
	svlMapping _reg2plane;
	svlSortVector<int,int> _regSizes;
	void computeHulls(vector<svlPoint3d> &Pts);
	void computeCameraPlanes(bool do_hulls = true);

public:
	// constructors
	svlPlaneClustering();
	svlPlaneClustering(int w, int h);
	svlPlaneClustering(int w, int h, svlCameraIntrinsics intrinsics, svlCameraExtrinsics extrinsics);
	void init();
	virtual ~svlPlaneClustering();

	void setInExtrinsics(int w_, int h_, svlCameraIntrinsics intrinsics, svlCameraExtrinsics extrinsics);
	void setCvMats(CvMat* X, CvMat* Y, CvMat* Z, CvMat* nX, CvMat* nY, CvMat* nZ, CvMat* Ix);
	void declareDefaultOptions();
	bool segmentRawPointCloud(vector<svlPoint3d> *inP);
	bool segmentSuperPointCloud(svlPointCloudData *pcData, CvMat *Ix);
	void drawHulls(CvMat *Hulls);
	void computeZHat(CvMat *ZHat);

	static vector<svlPlane> computeHulls(vector<svlPoint3d> &Pts, vector<vector<int> > &regionIx);
	
	void outputCameraPlanes(FILE* fout, bool do_hulls = true);

	// feature extractors
	vector<vector<double> > getRectangularFeatures(vector<CvRect> &rects);
	vector<double> getRectangularFeatures(int x, int y, int w, int h, FILE *fo = NULL);

	void Update();
};

