//TODO-- add full lasik header
//Author-- Ian Goodfellow <ia3n@cs.stanford.edu>
//Description: A renderable point cloud that exploits the fact that the points are sampled from a (possibly quite distorted) grid

#pragma once

#include "svlAbstractCloud.h"
#include "svlObjectList3d.h"

class svlStructuredCloud : public svlAbstractCloud
{

public:

  //Makes an empty cloud
  svlStructuredCloud();

  //Makes a deep copy of all matrices-- client is responsible for freeing them
  svlStructuredCloud(const CvMat * x, const CvMat *y, const CvMat * z, \
		     const CvMat * nx = NULL, const CvMat * ny = NULL, \
		     const CvMat * nz = NULL);

  //Makes a deep copy
  svlStructuredCloud(const svlStructuredCloud & o);
  
  //Makes a deep copy
  const svlStructuredCloud & operator=(const svlStructuredCloud & o);


  virtual ~svlStructuredCloud();

  virtual void render(
		      svlRenderType renderType,		      
		      double option = 0.1) const; //option can be, i.e., the max depth change per polygon for RENDER_STRUCTURED_SURFACE_PRETTY

  virtual unsigned size() const;
  virtual svlPoint3d getPoint(unsigned idx) const;

  svlObject3d boundingBox() const;

 protected:

  void copy(const svlStructuredCloud & o);
  void destroy();

 protected:

  bool _empty;

  CvMat * _x;
  CvMat * _y;
  CvMat * _z;
  CvMat * _nx;
  CvMat * _ny;
  CvMat * _nz;

  mutable CvMat * _bad;

};
